#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename:""" $HeadURL$
#
#  Version:""" $Revision$,
#           $Date$
#           $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
from ..GXIntEnum import GXIntEnum

class Unit(GXIntEnum):
    """Enumerated units."""
    #pylint: disable=too-few-public-methods

    #No Unit.
    NONE = 0
    #Year.
    YEAR = 1
    #Month.
    MONTH = 2
    #Week.
    WEEK = 3
    #Day.
    DAY = 4
    #Hour.
    HOUR = 5
    #Minute.
    MINUTE = 6
    #Second.
    SECOND = 7
    #Phase angle degree.
    PHASE_ANGLE_DEGREE = 8
    #Temperature
    TEMPERATURE = 9
    #Local currency.
    LOCAL_CURRENCY = 10
    #Length l meter m.
    LENGTH = 11
    #Speed.
    SPEED = 12
    #Volume V m3.
    VOLUME_CUBIC_METER = 13
    #Corrected volume m3.
    CORRECTED_VOLUME = 14

    VOLUME_FLUX_HOUR = 15
    CORRECTED_VOLUME_FLUX_HOUR = 16
    VOLUME_FLUXDAY = 17
    CORRECTE_VOLUME_FLUX_DAY = 18

    #Volume 10-3 m3.
    VOLUME_LITER = 19
    #Mass m kilogram kg.
    MASS_KG = 20
    #return "Force F newton N.
    FORCE = 21
    #Energy newtonmeter J = Nm = Ws.
    ENERGY = 22

    PRESSURE_PASCAL = 23

    PRESSURE_BAR = 24

    #Energy joule J = Nm = Ws.
    ENERGY_JOULE = 25

    THERMAL_POWER = 26

    #Active power.
    ACTIVE_POWER = 27

    #Apparent power S.
    APPARENT_POWER = 28

    #Reactive power Q.
    REACTIVE_POWER = 29

    #Active energy W6060s.
    ACTIVE_ENERGY = 30

    #Apparent energy VA6060s.
    APPARENT_ENERGY = 31

    #Reactive energy var6060s.
    REACTIVE_ENERGY = 32

    #Current I ampere A.
    CURRENT = 33

    #Electrical charge Q coulomb C = As.
    ELECTRICAL_CHARGE = 34

    #Voltage.
    VOLTAGE = 35

    ELECTRICAL_FIELD_STRENGTH = 36

    CAPACITY = 37

    RESISTANCE = 38

    #Resistivity.
    RESISTIVITY = 39

    #Magnetic flux F weber Wb = Vs.
    MAGNETIC_FLUX = 40

    INDUCTION = 41

    MAGNETIC = 42

    INDUCTIVITY = 43

    #Frequency f.
    FREQUENCY = 44

    ACTIVE = 45

    #Reactive energy meter constant.
    REACTIVE = 46

    #Apparent energy meter constant.
    APPARENT = 47

    #V26060s.
    V260 = 48

    #A26060s.
    A260 = 49

    MASS_KG_PER_SECOND = 50

    CONDUCTANCE = 51

    #Temperature in Kelvin.
    KELVIN = 52

    V2H = 53

    A2H = 54

    CUBIC_METER_RV = 55

    #Percentage.
    PERCENTAGE = 56

    #Ah ampere hours.
    AMPERE_HOURS = 57

    ENERGY_PER_VOLUME = 60

    WOBBE = 61

    #Mol % molar fraction of gas composition mole percent (Basic gas composition unit).
    MOLE_PERCENT = 62

    MASS_DENSITY = 63

    #Dynamic viscosity pascal second =Characteristic of gas stream).
    PASCAL_SECOND = 64

    JOULE_KILOGRAM = 65

    # Pressure, gram per square centimeter.
    PRESSURE_GRAM_PER_SQUARE_CENTIMETER = 66

    # Pressure, atmosphere.
    PRESSURE_ATMOSPHERE = 67

    # dBm Signal strength =e.g.  of GSM radio systems).
    SIGNAL_STRENGTH = 70
   
     # Signal strength dB microvolt.    
    SIGNAL_STRENGTH_MICRO_VOLT = 71
    
    # Logarithmic unit that expresses the ratio between two values of a physical quantity    
    DB = 72
    
    # Length in inches.
    INCH = 128
    
    # Foot (Length).    
    FOOT = 129
    
    # Pound (mass).
    POUND = 130
    
    # Fahrenheit.
    FAHRENHEIT = 131
    
    # Rankine.    
    RANKINE = 132
    
    # Square inch.
    SQUARE_INCH = 133
    
    # Square foot.
    SQUARE_FOOT = 134
    
    # Acre.
    ACRE = 135
    
    # Cubic inch.
    CUBIC_INCH = 136
    
    # Cubic foot.
    CUBIC_FOOT = 137
    
    # Acre foot.
    ACRE_FOOT = 138
    
    # Gallon (imperial).
    GALLON_IMPERIAL = 139
    
    #  Gallon (US).
    GALLON_US = 140
    
    # Pound force.
    POUND_FORCE = 141
    
    # Pound force per square inch.
    POUND_FORCE_PER_SQUARE_INCH = 142
    
    # Pound per cubic foot.
    POUND_PER_CUBIC_FOOT = 143
    
    # Pound per (foot second).
    POUND_PER_FOOT_SECOND = 144
    
    # Square foot per second.
    SQUARE_FOOT_PER_SECOND = 145
    
    # British thermal unit.
    BRITISH_THERMAL_UNIT = 146
    
    # Therm EU.
    THERM_EU = 147
    
    # Therm US.
    THERM_US = 148
    
    # British thermal unit per pound.
    BRITISH_THERMAL_UNIT_PER_POUND = 149
    
    # British thermal unit per cubic foot.
    BRITISH_THERMAL_UNIT_PER_CUBIC_FOOT = 150
    
    # Cubic feet.
    CUBIC_FEET = 151
    
    # Foot per second.
    FOOT_PER_SECOND = 152
    
    # Cubic foot per second.
    CUBIC_FOOT_PER_SECOND = 153
    
    # Cubic foot per min.
    CUBIC_FOOT_PER_MIN = 154
    
    # Cubic foot per hour.
    CUBIC_FOOT_PER_HOUR = 155
    
    # Cubic foot per day
    CUBIC_FOOT_PER_DAY = 156
    
    # Acre foot per second.
    ACRE_FOOT_PER_SECOND = 157
    
    # Acre foot per min.
    ACRE_FOOT_PER_MIN = 158
    
    #  Acre foot per hour.
    ACRE_FOOT_PER_HOUR = 159
    
    #  Acre foot per day.
    ACRE_FOOT_PER_DAY = 160
    
    # Imperial gallon.
    IMPERIAL_GALLON = 161
    
    # Imperial gallon per second.
    IMPERIAL_GALLON_PER_SECOND = 162
    
    # Imperial gallon per min.
    IMPERIAL_GALLON_PER_MIN = 163
    
    # Imperial gallon per hour.
    IMPERIAL_GALLON_PER_HOUR = 164
    
    # Imperial gallon per day.
    IMPERIAL_GALLON_PER_DAY = 165
    
    # US gallon.
    US_GALLON = 166
    
    # US gallon per second.
    US_GALLON_PER_SECOND = 167
    
    # US gallon per min.
    US_GALLON_PER_MIN = 168
    
    # US gallon per hour.
    US_GALLON_PER_HOUR = 169
    
    # US gallon per day.
    US_GALLON_PER_DAY = 170
    
    # British thermal unit per second.
    BRITISH_THERMAL_UNIT_PER_SECOND = 171
    
    # British thermal unit per minute.
    BRITISH_THERMAL_UNIT_PER_MIN = 172
    
    # British thermal unit per hour.
    BRITISH_THERMAL_UNIT_PER_HOUR = 173
    
    # British thermal unit per day.
    BRITISH_THERMAL_UNIT_PER_DAY = 174
   
    #Other Unit.
    OTHER_UNIT = 254

    #No Unit.
    NO_UNIT = 255



#
#  --------------------------------------------------------------------------
#   Gurux Ltd
#
#
#
#  Filename: $HeadURL$
#
#  Version: $Revision$,
#                   $Date$
#                   $Author$
#
#  Copyright (c) Gurux Ltd
#
# ---------------------------------------------------------------------------
#
#   DESCRIPTION
#
#  This file is a part of Gurux Device Framework.
#
#  Gurux Device Framework is Open Source software; you can redistribute it
#  and/or modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; version 2 of the License.
#  Gurux Device Framework is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#  See the GNU General Public License for more details.
#
#  More information of Gurux products: http://www.gurux.org
#
#  This code is licensed under the GNU General Public License v2.
#  Full text may be retrieved at http://www.gnu.org/licenses/gpl-2.0.txt
# ---------------------------------------------------------------------------
#
#  * Reserved for internal use.
#
class _GXFCS16:

    __fcs16Table = (0x0000, 0x1189, 0x2312, 0x329B, 0x4624,
                    0x57AD, 0x6536, 0x74BF, 0x8C48, 0x9DC1, 0xAF5A, 0xBED3, 0xCA6C,
                    0xDBE5, 0xE97E, 0xF8F7, 0x1081, 0x0108, 0x3393, 0x221A, 0x56A5,
                    0x472C, 0x75B7, 0x643E, 0x9CC9, 0x8D40, 0xBFDB, 0xAE52, 0xDAED,
                    0xCB64, 0xF9FF, 0xE876, 0x2102, 0x308B, 0x0210, 0x1399, 0x6726,
                    0x76AF, 0x4434, 0x55BD, 0xAD4A, 0xBCC3, 0x8E58, 0x9FD1, 0xEB6E,
                    0xFAE7, 0xC87C, 0xD9F5, 0x3183, 0x200A, 0x1291, 0x0318, 0x77A7,
                    0x662E, 0x54B5, 0x453C, 0xBDCB, 0xAC42, 0x9ED9, 0x8F50, 0xFBEF,
                    0xEA66, 0xD8FD, 0xC974, 0x4204, 0x538D, 0x6116, 0x709F, 0x0420,
                    0x15A9, 0x2732, 0x36BB, 0xCE4C, 0xDFC5, 0xED5E, 0xFCD7, 0x8868,
                    0x99E1, 0xAB7A, 0xBAF3, 0x5285, 0x430C, 0x7197, 0x601E, 0x14A1,
                    0x0528, 0x37B3, 0x263A, 0xDECD, 0xCF44, 0xFDDF, 0xEC56, 0x98E9,
                    0x8960, 0xBBFB, 0xAA72, 0x6306, 0x728F, 0x4014, 0x519D, 0x2522,
                    0x34AB, 0x0630, 0x17B9, 0xEF4E, 0xFEC7, 0xCC5C, 0xDDD5, 0xA96A,
                    0xB8E3, 0x8A78, 0x9BF1, 0x7387, 0x620E, 0x5095, 0x411C, 0x35A3,
                    0x242A, 0x16B1, 0x0738, 0xFFCF, 0xEE46, 0xDCDD, 0xCD54, 0xB9EB,
                    0xA862, 0x9AF9, 0x8B70, 0x8408, 0x9581, 0xA71A, 0xB693, 0xC22C,
                    0xD3A5, 0xE13E, 0xF0B7, 0x0840, 0x19C9, 0x2B52, 0x3ADB, 0x4E64,
                    0x5FED, 0x6D76, 0x7CFF, 0x9489, 0x8500, 0xB79B, 0xA612, 0xD2AD,
                    0xC324, 0xF1BF, 0xE036, 0x18C1, 0x0948, 0x3BD3, 0x2A5A, 0x5EE5,
                    0x4F6C, 0x7DF7, 0x6C7E, 0xA50A, 0xB483, 0x8618, 0x9791, 0xE32E,
                    0xF2A7, 0xC03C, 0xD1B5, 0x2942, 0x38CB, 0x0A50, 0x1BD9, 0x6F66,
                    0x7EEF, 0x4C74, 0x5DFD, 0xB58B, 0xA402, 0x9699, 0x8710, 0xF3AF,
                    0xE226, 0xD0BD, 0xC134, 0x39C3, 0x284A, 0x1AD1, 0x0B58, 0x7FE7,
                    0x6E6E, 0x5CF5, 0x4D7C, 0xC60C, 0xD785, 0xE51E, 0xF497, 0x8028,
                    0x91A1, 0xA33A, 0xB2B3, 0x4A44, 0x5BCD, 0x6956, 0x78DF, 0x0C60,
                    0x1DE9, 0x2F72, 0x3EFB, 0xD68D, 0xC704, 0xF59F, 0xE416, 0x90A9,
                    0x8120, 0xB3BB, 0xA232, 0x5AC5, 0x4B4C, 0x79D7, 0x685E, 0x1CE1,
                    0x0D68, 0x3FF3, 0x2E7A, 0xE70E, 0xF687, 0xC41C, 0xD595, 0xA12A,
                    0xB0A3, 0x8238, 0x93B1, 0x6B46, 0x7ACF, 0x4854, 0x59DD, 0x2D62,
                    0x3CEB, 0x0E70, 0x1FF9, 0xF78F, 0xE606, 0xD49D, 0xC514, 0xB1AB,
                    0xA022, 0x92B9, 0x8330, 0x7BC7, 0x6A4E, 0x58D5, 0x495C, 0x3DE3,
                    0x2C6A, 0x1EF1, 0x0F78)

    #
    #      * Reserved for internal use.
    #      *
    #      * @param buff
    #      * @param offset
    #      * @param count
    #      * @return
    #
    @classmethod
    def countFCS16(cls, buff, offset, count):
        fcs16 = 0xFFFF
        pos = offset
        while pos < offset + count:
            fcs16 = int((((fcs16 >> 8) ^ _GXFCS16.__fcs16Table[(fcs16 ^ buff[pos]) & 0xFF]) & 0xFFFF))
            pos += 1
        fcs16 = ~fcs16
        fcs16 = ((fcs16 >> 8) & 0xFF) | (fcs16 << 8)
        return fcs16 & 0xFFFF

    ___CRCPOLY = 0xD3B6BA00
    
    #Reserved for internal use.
    @classmethod
    def countFCS24(cls, buff, index, count):
        crcreg = 0
        j = 0
        while j < count:
            b = buff[index + j]
            i = 0
            while i < 8:
                crcreg >>= 1
                if (b and 0x80) != 0:
                    crcreg = crcreg or 0x80000000
                if (crcreg and 0x80) != 0:
                    crcreg = crcreg ^ cls.___CRCPOLY
                b <<= 1
                i = i + 1
            j = j + 1
        return crcreg >> 8
